import os

def make_dirs(path):
    os.makedirs(path, exist_ok=True)

def to_path(package):
    return package.replace(".", "/")

def write_file(path, content):
    with open(path, "w") as f:
        f.write(content)



from utils.file import make_dirs, to_path, write_file

def generate_entity(base_package, name, java_version, architecture):
    path = f"src/main/java/{to_path(base_package)}/domain/{name}/domain"
    make_dirs(path)
    file_path = f"{path}/{name}.java"

    content = f"""package {base_package}.domain.{name}.domain;

import jakarta.persistence.*;
import lombok.*;

@Entity
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Builder
public class {name} {{

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    private String name;
}}
"""
    write_file(file_path, content)



from utils.file import make_dirs, to_path, write_file
from utils.input import get_plural

def generate_controller(base_package, name, java_version, architecture):
    name_lower = name.lower()
    plural = get_plural(name)
    path_name = get_plural(name_lower)

    path = f"src/main/java/{to_path(base_package)}/domain/{name}/controller"
    make_dirs(path)
    file_path = f"{path}/{name}Controller.java"

    content = f"""package {base_package}.domain.{name}.controller;

import {base_package}.domain.{name}.dto.*;
import {base_package}.domain.{name}.service.{name}Service;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api/{path_name}")
public class {name}Controller {{

    private final {name}Service service;

    @PostMapping
    public ResponseEntity<{name}Response> create{name}(@RequestBody {name}Request request) {{
        return ResponseEntity.ok(service.create{name}(request));
    }}

    @GetMapping
    public ResponseEntity<List<{name}Response>> get{plural}() {{
        return ResponseEntity.ok(service.get{plural}());
    }}

    @GetMapping("/{{id}}")
    public ResponseEntity<{name}Response> get{name}ById(@PathVariable Long id) {{
        return ResponseEntity.ok(service.get{name}ById(id));
    }}

    @PutMapping("/{{id}}")
    public ResponseEntity<{name}Response> update{name}(@PathVariable Long id, @RequestBody {name}Request request) {{
        return ResponseEntity.ok(service.update{name}(id, request));
    }}

    @DeleteMapping("/{{id}}")
    public ResponseEntity<Void> delete{name}(@PathVariable Long id) {{
        service.delete{name}(id);
        return ResponseEntity.noContent().build();
    }}
}}
"""
    write_file(file_path, content)



from utils.file import make_dirs, to_path, write_file

def generate_repository(base_package, name, java_version, architecture):
    path = f"src/main/java/{to_path(base_package)}/domain/{name}/repository"
    make_dirs(path)
    file_path = f"{path}/{name}Repository.java"

    content = f"""package {base_package}.domain.{name}.repository;

import {base_package}.domain.{name}.domain.{name};
import org.springframework.data.jpa.repository.JpaRepository;

public interface {name}Repository extends JpaRepository<{name}, Long> {{
}}
"""
    write_file(file_path, content)



def get_input(prompt, default=None):
    if default:
        result = input(f"{prompt} [{default}]: \n> ")
        return result if result else default
    return input(f"{prompt}: \n> ")

def get_plural(word):
    """단어의 복수형을 반환합니다."""
    if word.endswith('y'):
        return word[:-1] + 'ies'
    elif word.endswith(('s', 'x', 'z', 'ch', 'sh')):
        return word + 'es'
    else:
        return word + 's'



from utils.file import make_dirs, to_path, write_file
from utils.input import get_plural

def generate_service_interface(base_package, name, java_version, architecture):
    name_plural = get_plural(name)
    
    if architecture == "hexagonal":
        path = f"src/main/java/{to_path(base_package)}/domain/{name}/service"
        make_dirs(path)
        file_path = f"{path}/{name}Service.java"
        content = f"""package {base_package}.domain.{name}.service;

import {base_package}.domain.{name}.domain.{name};
import {base_package}.domain.{name}.dto.{name}Request;
import {base_package}.domain.{name}.dto.{name}Response;

import java.util.List;

public interface {name}Service {{
    {name}Response create{name}({name}Request request);
    List<{name}Response> get{name_plural}();
    {name}Response get{name}ById(Long id);
    {name}Response update{name}(Long id, {name}Request request);
    void delete{name}(Long id);
}}
"""
    else:
        path = f"src/main/java/{to_path(base_package)}/domain/{name}/service"
        make_dirs(path)
        file_path = f"{path}/{name}Service.java"
        content = f"""package {base_package}.domain.{name}.service;

import {base_package}.domain.{name}.domain.{name};
import {base_package}.domain.{name}.dto.{name}Request;
import {base_package}.domain.{name}.dto.{name}Response;

import java.util.List;

public interface {name}Service {{
    {name}Response create{name}({name}Request request);
    List<{name}Response> get{name_plural}();
    {name}Response get{name}ById(Long id);
    {name}Response update{name}(Long id, {name}Request request);
    void delete{name}(Long id);
}}
"""
    write_file(file_path, content)

def generate_service_impl(base_package, name, java_version, architecture):
    name_plural = get_plural(name)
    
    if architecture == "hexagonal":
        path = f"src/main/java/{to_path(base_package)}/domain/{name}/service"
        make_dirs(path)
        file_path = f"{path}/{name}ServiceImpl.java"
        content = f"""package {base_package}.domain.{name}.service;

import {base_package}.domain.{name}.service.{name}Service;
import {base_package}.domain.{name}.domain.{name};
import {base_package}.domain.{name}.dto.{name}Request;
import {base_package}.domain.{name}.dto.{name}Response;
import {base_package}.domain.{name}.repository.{name}Repository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class {name}ServiceImpl implements {name}Service {{

    private final {name}Repository repository;

    @Override
    public {name}Response create{name}({name}Request request) {{
        {name} entity = {name}.builder()
                .name(request.name())
                .build();
        {name} saved = repository.save(entity);
        return new {name}Response(saved.getId(), saved.getName());
    }}

    @Override
    public List<{name}Response> get{name_plural}() {{
        return repository.findAll().stream()
                .map(entity -> new {name}Response(entity.getId(), entity.getName()))
                .collect(Collectors.toList());
    }}

    @Override
    public {name}Response get{name}ById(Long id) {{
        {name} entity = repository.findById(id)
                .orElseThrow(() -> new RuntimeException("{name} not found"));
        return new {name}Response(entity.getId(), entity.getName());
    }}

    @Override
    public {name}Response update{name}(Long id, {name}Request request) {{
        {name} entity = repository.findById(id)
                .orElseThrow(() -> new RuntimeException("{name} not found"));
        entity.setName(request.name());
        {name} updated = repository.save(entity);
        return new {name}Response(updated.getId(), updated.getName());
    }}

    @Override
    public void delete{name}(Long id) {{
        repository.deleteById(id);
    }}
}}
"""
    else:
        path = f"src/main/java/{to_path(base_package)}/domain/{name}/service"
        make_dirs(path)
        file_path = f"{path}/{name}ServiceImpl.java"
        content = f"""package {base_package}.domain.{name}.service;

import {base_package}.domain.{name}.domain.{name};
import {base_package}.domain.{name}.dto.{name}Request;
import {base_package}.domain.{name}.dto.{name}Response;
import {base_package}.domain.{name}.repository.{name}Repository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class {name}ServiceImpl implements {name}Service {{

    private final {name}Repository repository;

    @Override
    public {name}Response create{name}({name}Request request) {{
        {name} entity = {name}.builder()
                .name(request.name())
                .build();
        {name} saved = repository.save(entity);
        return new {name}Response(saved.getId(), saved.getName());
    }}

    @Override
    public List<{name}Response> get{name_plural}() {{
        return repository.findAll().stream()
                .map(entity -> new {name}Response(entity.getId(), entity.getName()))
                .collect(Collectors.toList());
    }}

    @Override
    public {name}Response get{name}ById(Long id) {{
        {name} entity = repository.findById(id)
                .orElseThrow(() -> new RuntimeException("{name} not found"));
        return new {name}Response(entity.getId(), entity.getName());
    }}

    @Override
    public {name}Response update{name}(Long id, {name}Request request) {{
        {name} entity = repository.findById(id)
                .orElseThrow(() -> new RuntimeException("{name} not found"));
        entity.setName(request.name());
        {name} updated = repository.save(entity);
        return new {name}Response(updated.getId(), updated.getName());
    }}

    @Override
    public void delete{name}(Long id) {{
        repository.deleteById(id);
    }}
}}
"""
    write_file(file_path, content)


from utils.file import make_dirs, to_path, write_file

def generate_dto(base_package, name, java_version, architecture):
    dto_path = f"src/main/java/{to_path(base_package)}/domain/{name}/dto"
    make_dirs(dto_path)

    # Request
    request_path = f"{dto_path}/{name}Request.java"
    if java_version >= 17:
        request_content = f"""package {base_package}.domain.{name}.dto;

public record {name}Request(
    String name
) {{
}}
"""
    else:
        request_content = f"""package {base_package}.domain.{name}.dto;

import lombok.*;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class {name}Request {{
    private String name;
}}
"""
    write_file(request_path, request_content)

    # Response
    response_path = f"{dto_path}/{name}Response.java"
    if java_version >= 17:
        response_content = f"""package {base_package}.domain.{name}.dto;

public record {name}Response(
    Long id,
    String name
) {{
}}
"""
    else:
        response_content = f"""package {base_package}.domain.{name}.dto;

import lombok.*;

@Data
@NoArgsConstructor
@AllArgsConstructor
public class {name}Response {{
    private Long id;
    private String name;
}}
"""
    write_file(response_path, response_content)
